@@ -106,12 +106,12 @@ function(file, ignoreBlanks = TRUE, handlers = NULL,
   if(!missing(handlers) & !as.logical(asTree))
     return(handlers)
 
-  if(length(class(ans)))
+  if(!isSchema && length(class(ans)))
     class(ans) = oldClass(class(ans))
 
   if(inherits(ans, "XMLInternalDocument"))
     addDocFinalizer(ans, addFinalizer)
-  else if(!getDTD) {
+  else if(!getDTD && !isSchema) {
        #??? is this a good idea.
      class(ans) = oldClass("XMLDocumentContent")
   } 
@@ -152,6 +152,24 @@ function(file, ignoreBlanks = TRUE, handlers=NULL,
 }
 
 
+setGeneric("getEncoding",
+function(obj, ...)
+{
+  standardGeneric("getEncoding")
+})
+
+setMethod("getEncoding", "XMLInternalDocument",
+            function(obj, ...) {
+              .Call("R_getDocEncoding", obj, PACKAGE = "XML")
+            })
+
+setMethod("getEncoding", "XMLInternalNode",
+            function(obj, ...) {
+              .Call("R_getDocEncoding", obj, PACKAGE = "XML")
+            })
+
+
+
 
 
 xmlValidity =
